;;;;--------------------------------------------------------------------------
;;;;
;;;;                Ein einfacher bottom-up Recognizer
;;;;
;;;;                        (breadth-first)
;;;;
;;;;--------------------------------------------------------------------------


;;;; File : bu-1.lsp

;;;; Waehle eine geeignete Syntax und ein geeignetes Lexikon:
;;;; (use :syntax syntax5 :lexikon lexikon5)

(defun recognize_bu (stack1 &optional (stack2 nil) (symbol `(,(startsymbol))))
" Ein einfacher bottom-up Recognizer ohne Backtracking (breadth-first).
  Zuerst wird die Menge aller von der gebenen Konfiguration aus erreichbaren 
  Konfigurationen berechnet (KONFIGURATIONEN). Wenn keine der beiden Abbruchbe-
  dingungen erfuellt ist, wird RECOGNIZE_BU mit allen Elementen aus
  KONFIGURATIONEN aufgerufen. "
  (let ((konfigurationen (berechne-konfigurationen stack1 stack2)))
    (cond ((and (endp stack1) 
          	(equal stack2 symbol))
           (print t))
       	  ((endp konfigurationen)
           (print nil))
       	  (t (dolist (x konfigurationen 'stop)
               (apply #'recognize_bu (append x (list symbol))))))))

(defun berechne-konfigurationen (stack1 stack2)
" Diese Funktion berechnet die Menge aller von der aktuellen Konfiguration
  aus erreichbaren Konfiguarationen. "
  (append (mapcar #'(lambda (x) (bu-reduce stack1 stack2 x))
            	  (matching-regeln stack2))
       	  (when (shift-p stack1 stack2)
          	(list (shift stack1 stack2)))))



(defun shift (stack1 stack2)
  (list (rest stack1)
	(cons (first stack1) stack2)))

(defun shift-p (stack1 stack2)
" Dieses Praedikat prueft, ob SHIFT angewendet werden kann oder nicht:
  Die Trennung von Syntax und Lexikon laesst die Anwendung von Shift nur
  dann sinnvoll erscheinen, wenn das erste Symbol von STACK2 kein Symbol
  der Eingabekette ist; d.h. nach jedem Shift wird zunaechst reduziert."
  (and stack1
       (or (endp stack2)
           (lexkat-p (first stack2))
           (synkat-p (first stack2)))))

(defun bu-reduce (stack1 stack2 regel)
  (list stack1
     	(cons (first regel)
              (nthcdr (1- (length regel)) stack2))))


(defun matching-regeln (stack2)
" Diese Funktion findet alle (syntaktischen/lexikalischen) Regeln, durch die
  die STACK2 reduziert werden kann. "
  (if (or (lexkat-p (first stack2))
       	  (synkat-p (first stack2)))
      (remove-if #'(lambda (x) (not (match x stack2))) (regeln))
      (let ((eintrag (lexikon-eintrag (first stack2))))
     	(mapcar #'(lambda (x) (append x (list (first eintrag))))
          	(rest eintrag)))))
          
(defun match (regel stack)
" Dieses Praedikat testet, ob die rechte Seite von REGEL mit den invertierten
  ersten Symbolen von STACK uebereinstimmen. "
  (let ((vergleiche (length (rest regel))))
    (dotimes (x vergleiche t)
      	     (unless (eq (nth x (rest regel))
            		 (nth (- vergleiche x 1) stack))
     		     (return nil)))))




 
