;;;;--------------------------------------------------------------------------
;;;;
;;;;			 Ein einfacher bottom-up Parser
;;;;
;;;;  			       (breadth-first)
;;;;
;;;;--------------------------------------------------------------------------


;;;; File : bu-2.lsp

;;;; Waehle eine geeignete Syntax und ein geeignetes Lexikon:
(use :syntax syntax5 :lexikon lexikon5)


;;; Hauptfunktionen
;;; ---------------

(defun parse_bu (stack1 &optional (stack2 nil) (stack3 nil)
			          (symbol `(,(startsymbol))))
" Ein einfacher bottom-up Parser ohne Backtracking (breadth-first).
  Zuerst wird die Menge aller von der gebenen Konfiguration aus erreichbaren 
  Konfigurationen berechnet (KONFIGURATIONEN). Wenn keine der beiden Abbruch-
  bedingungen erfuellt ist, wird PARSE_BU mit allen Elementen aus KONFIGURA-
  TIONEN aufgerufen. "
  (let ((konfigurationen (berechne-konfigurationen stack1 stack2 stack3)))
   (cond ((and (endp stack1) 
		(equal stack2 symbol)) 
	   (print (first stack3)))
	  ((endp konfigurationen)	
	   (print nil))
	  (t (dolist (x konfigurationen)
	       (apply #'parse_bu (append x (list symbol))))))))

(defun berechne-konfigurationen (stack1 stack2 stack3)
" Diese Funktion berechnet die Menge aller von der aktuellen Konfiguration
  aus erreichbaren Konfiguarationen. "
  (append (mapcar #'(lambda (x) (bu-reduce stack1 stack2 stack3 x))
		  (matching-regeln stack2))
	  (when (shift-p stack1 stack2)
		(list (shift stack1 stack2 stack3)))))

(defun shift (stack1 stack2 stack3)
  (list (rest stack1)
	(cons (first stack1) stack2)
	stack3))

(defun shift-p (stack1 stack2)
" Dieses Praedikat prueft, ob SHIFT angewendet werden kann oder nicht:
  Die Trennung von Syntax und Lexikon laesst die Anwendung von Shift nur
  dann sinnvoll erscheinen, wenn das erste Symbol von STACK2 kein Symbol
  der Eingabekette ist; d.h. nach jedem Shift wird zunaechst reduziert."
  (and stack1
       (or (endp stack2)
	   (lexkat-p (first stack2))
	   (synkat-p (first stack2)))))

	   
(defun bu-reduce (stack1 stack2 stack3 regel)
  (list stack1
	(cons (first regel)
	      (nthcdr (1- (length regel)) stack2))
	(transform stack3 regel)))


(defun transform (stack regel)
" Generiert in STACK eine Strukturbeschreibung."
  (let* ((rs-laenge (length (rest regel)))
	 (letztes-symbol (nth rs-laenge regel)))
    (if (or (> rs-laenge 1)			; syntaktische Regel:
	    (lexkat-p letztes-symbol)   	; Struktur bilden
	    (synkat-p letztes-symbol))
	(append (butlast stack rs-laenge)
		(list (cons (first regel)
			    (nthcdr (- (length stack) rs-laenge)
				    stack))))
	(append stack (list regel)))))		; lex. Regel: hinzufuegen



(defun matching-regeln (stack2)
" Diese Funktion findet alle (syntaktischen/lexikalischen) Regeln, durch die
  die STACK2 reduziert werden kann. "
  (if (or (lexkat-p (first stack2))
	  (synkat-p (first stack2)))
      (remove-if #'(lambda (x) (not (match x stack2))) (regeln))
      (let ((eintrag (lexikon-eintrag (first stack2))))
	(mapcar #'(lambda (x) (append x (list (first eintrag))))
		(rest eintrag)))))
		
(defun match (regel stack)
" Dieses Praedikat testet, ob die rechte Seite von REGEL mit den invertierten
  ersten Symbolen von STACK uebereinstimmen. "
  (let ((vergleiche (length (rest regel))))
    (dotimes (x vergleiche t)
      (unless (eq (nth x (rest regel))
		  (nth (- vergleiche x 1) stack))
	(return nil)))))

