;;;;--------------------------------------------------------------------------
;;;;
;;;;                Ein einfacher bottom-up Recognizer
;;;;                        mit Backtracking
;;;;
;;;;                         (depth-first)
;;;;
;;;;--------------------------------------------------------------------------


;;;; File : bu-3.lsp
;;;; letzte Aenderungen: 18.02.97

;;;; Geeignete Testdaten finden sich am Ende der Datei.
;;;; Es wurde eine gegenueber den anderen Programmen leicht modifizierte
;;;; Repraesentation fuer Syntax & Lexikon verwendet (s.u.).


;;; RECOGNIZE-BU/DF
;;; Hauptfunktion
;;; Nur der erste Rechtsparse wird gefunden.
(defun recognize_bu/df (stack1 &optional stack2 stack3 &aux konfiguration)
   (cond ((and (endp stack1) (equal stack2 (list (startsymbol)))) stack3)
         ((setq konfiguration (reducable stack1 stack2 stack3))
	  (apply #'recognize_bu/df konfiguration))
         ((not (endp stack1))
          (recognize_bu/df (rest stack1) 
                           (cons (first stack1) stack2)
                           (cons 0 stack3)))
         ((setq konfiguration (backtrack stack1 stack2 stack3))
          (apply #'recognize_bu/df konfiguration))))


(defun reducable (stack1 stack2 stack3 &optional (limit 0))                      
   (dolist (regel (regeln))
      (if (and (> (first (last regel)) limit)
               (equal (butlast regel 2) 
                      (butlast stack2 (- (length stack2) (- (length regel) 
                                                            2)))))
          (return (list stack1 (cons (first (nthcdr (- (length regel) 2) regel))
                              (nthcdr (- (length regel) 2) stack2))
                     (append (last regel) stack3))))))



(defun backtrack (stack1 stack2 stack3)
   (unless (endp stack3)
       (if (> (first stack3) 0)
           (backtrack-r stack1 
                        (append (butlast (regel-nr-n (first stack3)) 2)
                                (rest stack2))
                        (rest stack3) 
                        (first stack3))
           (backtrack-s stack1 stack2 stack3))))

(defun backtrack-r (stack1 stack2 stack3 limit)
   (cond ((reducable stack1 stack2 stack3 limit))
         ((not (endp stack1))
          (list (rest stack1)
                (cons (first stack1) stack2)
                (cons 0 stack3)))
         (t (backtrack stack1 stack2 stack3))))

(defun backtrack-s (stack1 stack2 stack3)
  (backtrack (cons (first stack2) stack1)
             (rest stack2)
             (rest stack3)))



;;; Selektoren


(defun startsymbol ()
  (third *syntax*))

(defun regeln ()
   (append (fourth *syntax*) *lexikon*))

(defun regel-nr-n (n)
   (rassoc (list n) (regeln) :test #'equal :key #'last))


;;; Testdaten

;;; alle Regeln der Syntax werden in folgender Form repraesentiert:
;;; (invertierte-rechte-Seite linke-Seite Regelindex)

(defvar *syntax*
   '((S np vp n1 PP)     ; nicht-terminales Vokabular
     (det adj n v p)     ; terminales Vokabular
     S                   ; Startsymbol
     ((vp np s 1)        ; 1: S -> NP VP
      (n1 np 2)          ; 2: NP -> N1
      (n1 det np 3)      ; 3: NP -> det N1
      (n n1 4)           ; 4: N1 -> n
      (n1 adj n1 5)      ; 5: N1 -> adj N1
      (pp n n1 6)        ; 6: N1 -> n PP
      (np p pp 7)        ; 7: PP -> p NP
      (v vp 8)           ; 8: VP -> v
      (np v vp 9))))     ; 9: VP -> v NP


;;; Format der Lexikoneintraege:
;;; (Wortform Kategorie Index)    

(defvar *lexikon*
   '((der det 10)        ; 10: det -> der
     (mann n 11)         ; 11: n -> mann
     (im p 12)           ; 12: p -> im
     (mond n 13)         ; 13: n -> mond
     (suchte v 14)       ; 14: v -> suchte
     (seine det 15)      ; 15: der -> seine
     (pfeife n 16)))     ; 16: n -> pfeife

