;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;                                                                     ;;;;;
;;;;;                                                                     ;;;;;
;;;;;                      Parsingtabellengenerator                       ;;;;;
;;;;;                             (CLR(1))                                ;;;;;
;;;;;                                                                     ;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; letzte nderung : 09.01.1997

;;; (A) Syntax
;;; ----------

(defclass syntax ()
  ((startsymbol :accessor startsymbol :initarg :s :initform 's :type symbol)
   (terminale   :accessor terminale-symbole :initarg :vt
		:initform nil :type list)
   (non-terminale :accessor nicht-terminale-symbole :initarg :vn 
		  :initform nil :type list)
   (regeln :accessor regeln :initarg :r :initform nil :type list)
   (erreichbarkeitstabelle :accessor tabelle :initarg :tabelle
			   :initform (make-hash-table :test #'eq)
			   :type hash-table)))


(defmethod initialisiere-syntax ((syntax syntax))
  (berechne-erreichbarkeitstabelle syntax))

(setq syntax1
      (make-instance 'syntax
		     :vn '(s c)
		     :vt '(d e)
		     :r '((s c c) (c d c) (c e)))
      syntax2
      (make-instance 'syntax
		     :vn '(s np vp pp)
		     :vt '(det n v p and)
		     :r '((s np vp) (s s pp) (s s and s)
			  (np n) (np det n) (np np pp) (np np and np)
			  (vp v np) (vp v s)
			  (pp p np)))
      syntax3
      (make-instance 'syntax
		     :vn '(s np n1 ap a1 pp srel np_rpro vp)
		     :vt '(det pro pn n adv adj p rpro v)
		     :r '((s np vp)
			  (np det n1) 
			  (np pro) (np pn) (np)
			  (n1 n) (n1 ap n1) (n1 n pp) (n1 n srel)
			  (ap adv a1) (ap a1) (a1 adj) (a1 adj a1)
			  (pp p np)
			  (srel np_rpro s)
			  (np_rpro rpro)
			  (vp v) (vp v np) (vp v np np) (vp v vp)))
      syntax4
      (make-instance 'syntax
		     :vn '(s np n1 ap a1 vp)
		     :vt '(det n adj p v)
		     :r '((s np vp)
			  (np det n1) 
			  (n1 n) (n1 ap n1)
			  (ap a1) (a1 adj) (a1 adj a1)
			  (vp v np))))



;;; (B) Erreichbarkeitsrelation FIRST* (nur Terminale/inkl. Tilgungsregeln)
;;; -----------------------------------------------------------------------

(defmethod berechne-erreichbarkeitstabelle ((syntax syntax))
  (loop for kategorie in (nicht-terminale-symbole syntax) do
	(setf (gethash kategorie (tabelle syntax))
	      (Remove-If-Not #'(Lambda (x)
				       (member x (terminale-symbole syntax)))
			     (erreichbar syntax kategorie)))))

;; Transitiver Abschlu der Relation DIREKT-ERREICHBAR
(defmethod erreichbar ((syntax syntax) symb
		       &optional (erreichbare-symbole `(,symb)))
   (let ((neue-symbole
	  (set-difference (direkt-erreichbar syntax symb)
			  erreichbare-symbole)))
     (dolist (symbol neue-symbole erreichbare-symbole)
       (push symbol erreichbare-symbole)
       (if (member symbol (nicht-terminale-symbole syntax))
	   (setq erreichbare-symbole
		 (union erreichbare-symbole
			(erreichbar syntax symbol erreichbare-symbole)))))))

;; Diese Prozedur berechnet fr ein Symbol die Menge aller direkt erreichbaren
;; Symbole (linke Ecken):
(defmethod direkt-erreichbar ((syntax syntax) symbol)
  (loop with resultat
	for regelseite in (expansionen syntax symbol)
	if (not regelseite)
	  do (pushnew regelseite resultat)
	else
	  do (loop for kategorie in regelseite
		   do (pushnew kategorie resultat)
		   while (member nil (expansionen syntax kategorie)))
	finally (return  resultat)))

(defmethod erreichbare-symbole ((syntax syntax) symbol)
  (or (gethash symbol (tabelle syntax))
      (list symbol)))

(defmethod expansionen ((g syntax) symbol)
  (apply #'append
	 (mapcar #'(lambda (x) (if (eq (first x) symbol) (list (rest x))))
		 (regeln g))))


;;; (C) Generierungsalgorithmus
;;; ---------------------------

(defmethod compute-lr-table ((syntax syntax))
  (let* ((item-list (compute-item-lists syntax))		     
	 (action (make-array (list (length item-list)
				   (+ (length (terminale-symbole syntax))
				      1))))
	 (goto (make-array (list (length item-list)
				 (length (nicht-terminale-symbole syntax))))))
    (action-tabelle syntax action item-list)
    (pprint action)
    (setq *a* action)
    (goto-tabelle syntax goto item-list)
    (pprint goto)
    (setq *g* goto)
    t))

(defmethod action-tabelle ((syntax syntax) (array array) kanten-mengen-liste)
  (loop	for kanten-menge in kanten-mengen-liste
	for x = 0 then (+ x 1) do
	(loop for kante in kanten-menge
	      if (member (first (offen kante))
			 (terminale-symbole syntax))
	      do (generiere-shift-eintrag array syntax (first (offen kante))
					  kanten-menge kanten-mengen-liste x)
	      else If (not (offen kante)) 
	              If (equal kante '(S1 (s) nil ($)))
		      do (pushnew 'acc
				  (aref array x
					(length (terminale-symbole syntax))))
                      Else
		         Do (generiere-reduce-eintrag array syntax kante x))))

(defmethod generiere-reduce-eintrag ((array array) (syntax syntax) kante zeile)
  (mapcar #'(lambda (y)
	      (pushnew (list 'r (index (cons (links kante) (closed kante))
				       (regeln syntax)
				       1))
		       (aref array
			     zeile
			     (index y (append (terminale-symbole syntax)
					      (list '$))))
		       :test #'equal))
	  (look-ahead kante)))

(defmethod generiere-shift-eintrag ((array array) (syntax syntax) symbol
				    kanten-menge kanten-mengen-liste zeile)
  (pushnew `(s ,(index (goto-edges syntax kanten-menge symbol)
		       kanten-mengen-liste))
	   (aref array zeile (index symbol (append (terminale-symbole syntax)
						   (list '$))))
	   :test #'equal))



(defmethod goto-tabelle ((syntax syntax) (array array) kanten-mengen-liste)
  (loop for kanten-menge in kanten-mengen-liste
	for x = 0 then (+ x 1)  do
	(loop for symbol in (nicht-terminale-symbole syntax)
	      for index = (index (goto-edges syntax kanten-menge symbol)
				 kanten-mengen-liste)
	      If index
	      do (push index
		       (aref array x (index
				      symbol
				      (nicht-terminale-symbole syntax)))))))

(defmethod compute-item-lists ((syntax syntax))
  (loop with resultat = `(,(closure syntax
				    `((s1 () (,(startsymbol syntax)) ($)))))
	with arbeit = resultat
	and kanten-menge
	while arbeit do
	(setq kanten-menge (first arbeit)
	      arbeit (rest arbeit))
	(loop for symbol in (append (terminale-symbole syntax)
				    (nicht-terminale-symbole syntax))
	      for neue-kanten-menge = (goto-edges syntax kanten-menge symbol)
	      if (and neue-kanten-menge
		      (not (member neue-kanten-menge resultat :test #'equal)))
	      do (setq resultat (append resultat (list neue-kanten-menge))
		       arbeit (append arbeit (list neue-kanten-menge))))
	finally (return resultat)))

(defmethod closure ((syntax syntax) kanten-menge)
  (loop with resultat = kanten-menge
        while kanten-menge do
	(setq kante (first kanten-menge)
	      kanten-menge (rest kanten-menge))
	(loop for expansion in (expansionen syntax (first (offen kante)))
	      for neue-kante = (list (first (offen kante))
				     nil
				     expansion
				     (if (rest (offen kante))
					 (gethash (second (offen kante))
						  (tabelle syntax))
				         (look-ahead kante)))
	      if (not (member neue-kante resultat :test #'equal)) do
	         (push neue-kante resultat)
		 (push neue-kante kanten-menge))
	finally (return resultat)))

(defmethod goto-edges ((syntax syntax) kanten-menge symbol)
  (loop with resultat
        for kante in kanten-menge
	If (eq (first (offen kante)) symbol)
	Collect (list (links kante)
		      (append (closed kante) (list symbol))
		      (rest (offen kante))
		      (look-ahead kante))
	into resultat
	finally (return (closure syntax resultat))))
	
;; Hilfsfunktionen:
;; ----------------						  

(defun links (kante)
  (first kante))
(defun closed (kante)
  (second kante))
(defun offen (kante)
  (third kante))
(defun look-ahead (kante)
  (fourth kante))

(defun index (objekt objekt-liste &optional (start 0))
  (loop for obj in objekt-liste
	for X = start then (+ x 1)
	if (equal obj objekt)
	return x))


(defun sac (a1 a2)
     (loop for x from 0 to (- (first (array-dimensions a1)) 1) do
       (loop for y from 0 to (- (second (array-dimensions a1)) 1) do
         (if (not (equal (aref a1 x y)
			 (aref a2 x y)))
            (return (format t "~a - ~a--" x y))))))