;----------------------------------------------------------------------
;
;            Daten zu fuer die Recognizer & Parser
;
;----------------------------------------------------------------------

; File: daten.lsp

(defvar *syntax* nil
  "Systemvariable fuer die aktuelle Syntax")

(defvar *lexikon* nil
  "Systemvariable fuer das aktuelle Lexikon")


(defvar *lexika* "lexikon1/2/...9")

(defvar *syntaxen* "syntax11/12/21/22/23/3/4/5/6/7/8/9")


(defun use (&key syntax lexikon)
"Funktion zum aktualisieren der Variablen *SYNTAX* & *LEXIKON*."
  (cond ((and syntax lexikon)
	 (setf *syntax* syntax *lexikon* lexikon))
	(syntax (setf *syntax* syntax))
	(lexikon (setf *lexikon* lexikon))
	(t (print *syntaxen*) *lexika*)))
	  

;;;; Test-Daten aus TD-TOOLS.LSP

(setf syntax11
      '((S NP VP N1 AP)
        (DET N A V)
        S
        ((S NP VP)
         (S NP NP)
         (NP)           ; Tilgungsregel: NP -> e
         (NP DET A N)
         (NP DET N1)
         (N1 AP N)
         (NP DET N)
         (AP A)
         (VP V))))

(setf syntax12
      '((S NP VP N1 AP)
        (DET N A V)
        S
        ((S NP VP)
         (NP DET A N)
         (NP DET N)
         (AP A)
         (VP V))))

(setf lexikon1
      '((der   (det) (n))
        (mann  (n))
        (kind  (n))
        (starb (v))
        (alte  (a))
        (neue  (a))))


;;;; Testdaten fuer die earley-basierten Recognizer und Parser

(setf syntax21
      '((S NP VP N1 AP)
        (DET N A V)
        S
        ((S NP VP) (S NP VP PP) (S S KONJ S)
         (NP DET N1) (NP N2 N) (NP det n) (np det a n) (N1 A N) (N2 det a)
         (VP V adv))))


(setf syntax22
      '((S NP VP N1 AP)
        (DET N A V)
        S
        ((S NP VP) (S NP VP PP) (S S KONJ S)
	 (PP)
         (NP DET N1) (NP N2 N) (NP det a n) (N1 A N) (N2 det a)
	 (VP V1 adv) (V1 v) (VP V adv))))


(defvar *deleteable*
  "Diese Systemvariable enthaelt die tilgbaren Symbole der Syntax22.
   Sie wird fuer die earley-basierten Parser mit Vorwaertsverkettung
   bentigt.")

(setf *deleteable*
      '(pp s))


(setf syntax23
      '((S NP VP N1 AP)
        (DET N A V)
        S
        ((S NP VP) (S NP VP PP) (S S KONJ S) (PP)
	 (NP DET A) (VP n v adv)
	 (NP DET N1) (NP det a n) (NP N2 N) (N1 A N) (N2 det a)
	 (VP V1 adv) (V1 v) (VP V adv))))


(setf lexikon2
      '((der   (det) (n)) (mann  (n)) (kind  (n))
        (starb (v)) (heute (adv))
        (alte  (a)) (neue  (a))))


;; L = cdcdcd/n
(setf syntax3
      '((s a b) (a b) s ((s) (s s a b a b a b))))

(setf lexikon3
      '((c (a)) (d (b))))


;; L = x/n
;;			 !!! WARNUNG !!!
;;
;; Die Variable *DELETEABLE*, in der die tilgbaren Symbole der Syntax ge-
;; speichert sind (earl-p-4.lsp), darf nicht das Symbol S enthalten, da
;; sonst jeder Satz aus L unendlich ambig ist (=> Stack overflow=
(setf syntax4
      '((s) (s) s ((s s s))))

(setf lexikon4 
      '((x (s))))


;;; Testdaten fuer bu-1.lsp und bu-2.lsp

(setf syntax5
     '((S NP VP N1)
       (det adj n v)
       S
       ((s np vp) (np det n1) (n1 n) (n1 adj n1)
	(vp v) (vp v np))))

(setf lexikon5
	'((die (det))
	  (ein (det))
	  (alte (adj))
	  (frau (n))
	  (lied (n))
	  (singt (v))))


;;; Testdaten fuer lc-1.lsp & lc-2.lsp

(setf syntax6
      '((s np vp pp) (p n v) s 
        ((s np vp) (np n) (vp v) (vp v np pp) (pp p np))))

(setf lexikon6
      '((eva (n)) (adam (n)) (schlug (v)) (aufs (p)) (maul (n))))


;;; Testdaten fuer cky-1.lsp cky-2.lsp

(setf syntax7
      '((s x y a b) (a b) s
        ((s x x) (s y b) (x x x) (x a b) (y b a) (y y b))))

(setf lexikon7
      '((c (a)) (d (b))))


;;;; Testdaten fuer idlp-r-1.lsp, idlp-r-2.lsp, idlp-p-1.lsp & idlp-p-2.lsp

(setf syntax8
  '( (S X Y Z) 
     (A B C) 
     S 
     ((S x y z) (x a) (y b) (z c)) 	; ID-Regeln
     ((y z)) ))				; LP-Regeln

(setf lexikon8
      '((c (a)) (d (b)) (e (c))))

(setf syntax9
      '((s) (a b c d e) S ((s a b c d e)) ()))

(setf lexikon9
      '((v (a))
	(w (b))
	(x (c))
	(y (d))
	(z (e))
	(k (a b c d e))))

;;;; weitere Syntaxen

(setf syntax10
      '((s np vp pp) 
	(n p det v)
	s
	((s np vp) (np n) (np det n) (np np pp)
		   (vp v np) (vp v np pp) (pp p np))))
(setf lexikon10
      '((obelix (n))
	(roemer (n))
	(wald (n))
	(fernrohr (n))
	(sah (v))
	(die (det))
	(im (p))
	(mit (p))
	(dem (det))))


;;;; Syntaxen fuer error-correcting Parser
;;;; ------------------------------------


;;; A. "Covering Grammar" fuer die Syntax G =
;;;      '((S NP VP)
;;;        (DET N A V)
;;;        S
;;;        ((S NP VP)
;;;         (NP DET A N)
;;;         (VP V)
;;;         (VP v NP)))

(setf syntax11
      '((S1 H I S NP VP Edet En Ea Ev)
	(det a n v)
	S1
	((S1 S) (S1 S H) (H H I) (H I) (S NP VP)
	 (NP Edet Ea En) (VP Ev) (VP Ev NP)
	 (Edet det) (Edet n) (Edet a) (Edet v) (Edet H det) (I det) (Edet)
	 (En n) (En det) (En a) (En v) (En H n) (I n) (En)
	 (Ea a) (Ea det) (Ea n) (Ea v) (Ea H a) (I a) (Ea)
	 (Ev v) (Ev det) (Ev n) (Ev a) (Ev H v) (I v) (Ev))))
   