;;;;---------------------------------------------------------------------------
;;;;
;;;;                  Ein earley-basierter Parser
;;;;
;;;;---------------------------------------------------------------------------


;;;;                    INFOS & INITIALISIERUNG
;;;;                    -----------------------

;;;; File : earl-p-1.lsp
;;;; Generierung der Chart durch Vorwaertsverkettung
;;;; keine Verarbeitung von Tilgungsregeln

;;;; Waehle eine geeignete Syntax und ein geeignetes Lexikon:
(use :syntax syntax21 :lexikon lexikon2)



;;;;                        PROGRAMM
;;;;                        --------

(defun parse_ear (satz &key (symbol (startsymbol)))
" Earley-basierter Recognizer fuer einfache kontextfreie Syntaxen."
  (dolist (kante (generiere-chart satz symbol) 
		(format t "~%Keine weiteren Parse gefunden !"))
    (when (and (passive-kante-p kante)
	       (eq (kopf kante) symbol)
	       (eq (start kante) 0)
	       (eq (ende kante) (length satz)))
          (pprint (cons (kopf kante) (struktur kante))))))


(defun generiere-chart (satz symbol &optional chart)
" Hauptfunktion zum Aufbau der Chart."
  (declare (special chart))
  (initialisiere-chart symbol)
  (dotimes (i (length satz) chart)
    (dolist (kante (lexikalische-kanten i (nth i satz)))
      (build kante))))


;;; Initialisierung der Chart

(defun initialisiere-chart (symbol)
" Generiert die Kanten fuer das Feld Chart(0, 0)."
  (dolist (regel (regeln))
    (when (eq (first regel) symbol)
          (build `((0 0) ,(first regel) nil ,(rest regel) nil)))))

(defun lexikalische-kanten (start wort)
" Generiert fuer ein Wort alle moeglichen lexikalischen Kanten; d.h. eine
  pro zugeordneter lexikalischer Kategorie."
  (mapcar #'(lambda (x)
	      (list (list start (+ start 1)) x nil nil (list wort)))
	  (eintrag-kategorien wort)))


;;; Berechnung der Chart

(defun build (kante)
" 'Arbeitspferd' das fuer den Aufbau der Chart sorgt.
  Nimmt eine Kante als Argument.
  Wenn KANTE noch nicht in der Chart enthalten ist, wird sie eingetragen.
  Ist sie aktiv, wird EXPAND aufgerufen; sonst COMPLETE."
  (declare (special chart))
  (when (neue-kante-p kante chart)
    (push kante chart)
    (if (aktive-kante-p kante)
        (expand kante)
        (complete kante))))


(defun expand (kante)
" EXPAND nimmt eine aktive Kante als Argument und generiert fuer jede Regel der
  Syntax mit der das naechste Symbol der aktiven Seite desrKanten expandiert
  werden kann, eine neue Kante."
  (dolist (x (expansionen (first (fourth kante))))
    (build (list (list (ende kante) (ende kante)) (first (offen kante)) nil x nil))))


(defun complete (pkante)
" COMPLETE nimmt als Argument eine passive Kante und traegt in die Chart
  diejenigen Kanten ein, die sich durch Verbindung dieser Kanten mit einer
  aktiven Kante der Chart bilden lassen." 
  (declare (special chart))
  (dolist (akante chart)
    (when (matching-kanten-p akante pkante)
          (build (bilde-kante akante pkante)))))

;;; Selektoren, Konstruktoren & Praedikate

;;;---------------------------------------

;;; Selektoren

(defun start (kante)
  (caar kante))

(defun ende (kante)
  (cadar kante))

(defun kopf (kante)
  (second kante))

(defun geschlossen (kante)
  (third kante))

(defun offen (kante)
  (fourth kante))

(defun struktur (kante)
  (fifth kante))


;;; Praedikate

(defun aktive-kante-p (kante)
" Das Praedikat testet, ob KANTE ein aktives Kante ist."
  (offen kante))

(defun passive-kante-p (kante)
" Das Praedikat testet, ob KANTE ein passives Kante ist."
  (not (offen kante)))

(defun matching-kanten-p (kante1 kante2)
" Das Praedikat testet, ob KANTE1 mit KANTE2 zu einem neuen Kante verbunden werden
  kann."
  (and (aktive-kante-p kante1)
       (eql (ende kante1) (start kante2))
       (eq  (first (offen kante1)) (kopf kante2))))

(defun neue-kante-p (kante chart)
" Das Praedikat testet, ob der Kante schon in der Chart enthalten ist
  oder nicht."
  (not (member kante chart :test #'equal)))


; Kantenkonstruktor

(defun bilde-kante (akante pkante)
  (list (list (start akante) (ende pkante))
	(kopf akante)
	(append (geschlossen akante) (list (kopf pkante)))
	(rest (offen akante))
	(append (struktur akante)
		(list (cons (first (offen akante)) (struktur pkante))))))
