;;;; --------------------------------------------------------------------------
;;;;
;;;;               Funktionen zum Verarbeiten von ID/LP-Syntaxen
;;;;
;;;; --------------------------------------------------------------------------

;;;; File: if-tools.lsp

(defun zulaessige-symbole (liste)
" ZULAESSIGE-SYMBOLE stellt fest, welche Symbole aus LISTE den anderen Symbolen
  aus LISTE vorangehen duerfen. "
  (let ((resultat nil))
    (dolist (x (remove-duplicates liste) resultat)
      (when (kann-vorangehen-p x (entferne x liste))
	(push x resultat)))))

(defun kann-vorangehen-p (symbol liste)
" KANN-VORANGEHEN prueft, ob SYMBOL den Symbolen aus Liste vorangehen darf. "
    (dolist (x liste t)
      (when (member (list x symbol) (lp-regeln) :test #'equal)
	(return nil))))

(defun entferne (object liste)
" ENTFERNE entfernt das erste Vorkommen von Objekt aus LISTE. "
  (remove object liste :count 1))



(defun zulaessige-regeln (open)
" ZULAESSIGE-REGELN berechnet die Menge der Regeln, die zur Bildung neuer zyklischer
  Kanten verwendet werden koennen. "
  (let ((symbole (zulaessige-symbole open))
	(resultat nil))
    (dolist (x symbole resultat)
      (setf resultat
	    (append resultat (matching-regeln x))))))

(defun matching-regeln (symbol)
"  MATCHING-REGELN generiert die Liste aller Regeln, deren linke Seite mit 
   Symbol identisch ist. "
  (remove-if-not #'(lambda (x) (eq symbol (first x))) (id-regeln)))



;;;; Neue Selektoren fuer ID/LP-Syntaxen:

(defun id-regeln ()
  (regeln))

(defun lp-regeln ()
  (fifth *syntax*))

;;;;---------------------------------------------------------------------------

;;; Konvertierung von ID/LP-Regeln in kontextfreie Regeln
;;; -----------------------------------------------------

(defun generiere_ps-regeln ()
  (let ((resultat nil))
    (dolist (id-regel (id-regeln) resultat)
      (setf resultat
	    (append (mapcar #'(lambda (x) (cons (first id-regel) x))
			    (lp-permutationen (rest id-regel)))
		    resultat)))))


(defun permutation (liste)
;; liefert die Liste mit allen moeglichen Permutationen von LISTE
;; Voraussetzung: kein Objekt kommt in LISTE mehrfach vor
  (if (endp (rest liste))
      (list liste)
    (let ((resultat nil))
      (dolist (x liste resultat)
	(setf resultat
	      (append (mapcar #'(lambda (y)
				  (cons x y))
			      (permutation (remove x liste)))
		      resultat))))))

(defun lp-permutationen (liste)
;; liefert die Liste mit allen moeglichen Permutationen von LISTE
;; Voraussetzung: kein Objekt kommt in LISTE mehrfach vor
  (if (endp (rest liste))
      (list liste)
    (let ((resultat nil))
      (dolist (x liste resultat)
	(dolist (y (lp-permutationen (remove x liste)))
	  (if (kann-vorangehen-p x y)
	      (push (cons x y) resultat)))))))


