
; file:  is-tools.lsp

; Funktionen zur Bestimmung der Inseln:

(defun inseln (satz)
" Diese Funktion erlaubt es dem Benutzer, wahlweise Worte des Satzes oder
  lexikalische Kategorien als Inseln fuer die Analyse zu markieren, bzw. mit
  der default-Inselkategorie (V) zu arbeiten. "
  (let ((antwort nil))
    (format t "~%Waehlen Sie eine der folgenden Optionen:~%")
    (format t "    (A)  -  dialoggesteuerte Festlegung von Wort-Inseln~%")
    (format t "    (B)  -  dialoggesteuerte Festlegung von Insel-Kategorien~%")
    (format t "    (C)  -  Verwendung der Kategorie V als Insel.~%> ")
    (setf antwort (read))
    (cond ((eq antwort 'a) (wort-inseln satz))
	  ((eq antwort 'b) (insel-kategorien satz))
	  (t (insel-kategorien satz '(V) )))))

(defun wort-inseln (satz)
" Diese Funktion legt durch einen mit dem Benutzer gefuehrten Dialog fest,
  welche Worte des Satzes als Insel verwendet werden."
  (let ((z 0) (heads nil) (noheads nil))
    (dolist (x satz (list heads noheads))
      (format t "~%Soll / ~A / als Insel verwendet werden?   " (nth z satz))
      (if (y-or-n-p)
	  (setf heads (append heads (lexikalische-kanten z x)))
	  (setf noheads (append noheads (lexikalische-kanten z x))))
      (setf z (1+ z)))))

(defun insel-kategorien (satz &optional default)
" Diese Funktion legt durch einen mit dem Benutzer gefuehrten Dialog fest,
  welche lexikalische Kategorien als Insel verwendet werden."
  (let ((z 0) (kategorien nil) (heads nil) (noheads nil))
    (unless (setf kategorien default)
      (format t "~%Welche der folgenden lexikalischen Kategorien:  ~{~S ~}~%" 
	        (lexikalische-kategorien))
      (format t "sollen als Insel verwendet werden? ~%>  ")
      (setf kategorien (read)) (terpri))
    (dolist (x satz (list heads noheads))
      (dolist (y (lexikalische-kanten z x))
	(if (member (kopf y) kategorien) 
	    (push y heads)
	    (push y noheads)))
      (setf z (1+ z)))))


(defun matching-kanten-p (kante1 kante2)
" Das Praedikat testet, ob KANTE1 mit KANTE2 zu einer neuen KANTE verbunden
   werden kann."
  (if (rechts kante1)
      (and (eql (ende kante1) (start kante2))
	   (eq (first (rechts kante1)) (kopf kante2)))
      (and (eql (start kante1) (ende kante2))
	   (eq (first (last (links kante1))) (kopf kante2)))))

(defun neue-kante-p (kante chart)
" Das Praedikat testet, ob der KANTE schon in der Chart enthalten ist
  oder nicht."
  (not (member kante chart :test #'equal)))

(defun aktive-kante-p (kante)
" Das Praedikat testet, ob KANTE ein aktives KANTE ist."
  (or (links kante) (rechts kante)))

(defun passive-kante-p (kante)
" Das Praedikat testet, ob KANTE ein passives KANTE ist."
  (not (aktive-kante-p kante)))


(defun start (kante)
  (first kante))

(defun ende (kante)
  (second kante))

(defun kopf (kante)
  (third kante))

(defun geschlossen (kante)
  (fifth kante))

(defun links (kante)
  (fourth kante))

(defun rechts (kante)
  (sixth kante))
