;;;;---------------------------------------------------------------------------
;;;;
;;;;                              LR-Recognizer
;;;;
;;;;---------------------------------------------------------------------------


;;;; file : lr-parsing


(defvar *syntax*)
(defvar *lexikon*)

(setf *syntax*
      '((S NP VP PP) (det n v prep) S 
	((S NP VP) (S S PP) (NP n) (NP det n) (NP NP PP) 
	 (PP prep NP) (VP v NP))))

(setf *lexikon*
      '((i n)
	(saw v)
	(a det)
	(man n)
	(with prep)
	(telescope n)))


(defvar *action*)
(defvar *goto*)

(setf *action*
      '((0 (det (s 3)) (n (s 4)))
	(1 (prep (s 6)) ($ accept))
	(2 (v (s 7)) (prep (s 6)))
	(3 (n (s 10)))
	(4 (v (r 3)) (prep (r 3)) ($ (r 3)))
	(5 (prep (r 2)) ($ (r 2)))
	(6 (det (s 3)) (n (s 4)))
	(7 (det (s 3)) (n (s 4)))
	(8 (prep (r 1)) ($ (r 1)))
	(9 (v (r 5)) (prep (r 5)) ($ (r 5)))
	(10 (v (r 4)) (prep (r 4)) ($ (r 4)))
	(11 (v (r 6)) (prep (r 6) (s 6)) ($ (r 6)))
	(12 (prep (r 7) (s 6)) ($ (r 7)))))

(setf *goto*
      '((0  (np 2) (s 1))
	(1  (pp 5))
	(2  (pp 9) (vp 8))
	(6  (np 11))
	(7  (np 12))
	(11 (pp 9))
	(12 (pp 9))))


;;;; ACTION
;;;; Selektorfunktion fuer den Action-Teil der Parsingtabelle
(defmacro action (zustand symbol)
  `(second (assoc ,symbol (rest (assoc ,zustand *action*)))))


;;;; GOTO
;;;; Selektorfunktion fuer den Goto-Teil der Parsingtabelle
(defmacro goto (zustand symbol)
  `(second (assoc ,symbol (rest (assoc ,zustand *goto*)))))


;;;; LR-SHIFT
;;;; Ausfuehren einer Shift-Operation
(defmacro lr-shift (stack zustand symbol)
  `(cons ,zustand (cons ,symbol ,stack)))

;;;; LR-REDUCE
;;;; Ausfuehren einer Reduce-Operation
(defmacro lr-reduce (stack regel-index)
  `(let* ((regel (nth (1- ,regel-index) ',(regeln)))
 	  (r-stack (nthcdr (* 2 (length (rest regel))) ,stack)))
     (cons (goto (first r-stack) (first regel))
	   (cons (first regel) r-stack))))
	 
;;;; KATEGORIE
;;;; Liefert als Wert die lexikalische Kategorie des naechsten Wortes der Eingabe
(defmacro kategorie (wort)
  `(if (eq ,wort '$)
       '$
       (second (assoc ,wort *lexikon*))))


;;;; RECOGNIZE_LR
;;;; Hauptfunktion
(defun recognize_lr (satz)
  (let* ((stack '(0))
	 (eingabe (append satz (list '$)))
	 (symbol nil)
	 (wert nil))
    (loop
      (setq symbol  (kategorie (first eingabe))
	    wert    (action (first stack) symbol))
       (print stack)
       (cond ((not wert) (return nil))
	     ((eq wert 'accept) (return t))
	     ((eq (first wert) 's) 
	      (setq stack    (lr-shift stack (second wert) symbol)
		    eingabe  (rest eingabe)))
	     ((eq (first wert) 'r)
	      (setq stack (lr-reduce stack (second wert))))))))



