;-----------------------------------------------------------------------------
;
;                           Strukturgenerierung
;                                  fuer
;                               Chart-Parser
;                      mit impliziter Repraesentation
;
;-----------------------------------------------------------------------------

;;;; File : pointer.lsp
;;;; Parser : earl-p-3.lsp / earl-p-4.lsp / earl-p-5.lsp
;;;;          idlp-p-2.lsp
;;;;          inselp-2.lsp / inselp-3.lsp



(defun auswerten (chart symbol satz)
" Diese Funktion generiert alle Strukturbeschreibungen fuer alle Kanten vom
  Typ SYMBOL, die sich ueber den ganzen Satz erstrecken."
   (dolist (kante chart (format t  "~%~%---------  Ende  ---------"))
      (when (and (passive-kante-p kante)
  	         (eq (kopf kante) symbol)
	         (eq (start kante) 0)
	         (eq (ende kante) (length satz)))
	(mapcar #'(lambda (x)
		    (pprint (generiere-struktur 
			      (append (butlast kante) (list (list x))) chart)))
		(pointer kante)))))


(defun generiere-struktur (kante chart)
" Generiert aus der Chart eine Strukturbeschreibung in der alle Lesarten des
  Satzes 'gepackt' repraesentiert werden (lokal ambiguity packing).
  Gepackte Konstituenten sind durch doppelte Klammerung und Identitaet des
  ersten Symbols der beiden Teilstrukturen erkennbar.
  Beispiel:   ... ((NP (NP1 ...) ...) (NP (NP2 ...) ...) ...) ...  "
  (cond ((endp (pointer kante))
	 (when (passive-kante-p kante) (list (kopf kante))))
	((symbolp (first (pointer kante))) (cons (kopf kante) (pointer kante)))
	((minusp (caar (pointer kante)))
	 (mapcar #'(lambda (X)	 
		     (generiere-struktur (nth X chart) chart))
		 (cadar (pointer kante))))
	(t (verarbeite (pointer kante) kante chart))))


(defun verarbeite (verweise kante chart)
" Generiert fuer jeden Eintraege der Pointer-Liste eine korrespondierende
  Strukturbeschreibung. Zwei Faelle sind zu unterscheiden:
  (a). (p1 (p2))        ==> keine direkte lokale Ambiguitaet
  (b). (p1 (p2 ... pn)) ==> direkte lokale Ambiguitaet."
  (let ((resultat nil))
    (dolist (point verweise
		   (if (rest resultat) (list resultat) (first resultat)))
      (push (ergaenze-struktur kante
			    (generiere-struktur (nth (first point) chart)
						chart)
			    (if (rest (second point))
				(mapcar #'(lambda (x)
					    (generiere-struktur (nth x chart) 
								chart))
					(second point))
				(generiere-struktur (nth (first(second point)) 
							 chart)
					  chart)))
	    resultat))))


(defun ergaenze-struktur (kante struktur1 struktur2)
" Wenn KANTE eine aktive Kante ist, wird nur die der Kante korrespondierende
  Struktur zurueckgegeben; sonst muss auch die Kopf-Kategorie von KANTE in
  die Struktur aufgenommen werden."
  (let ((struktur (kombiniere-strukturen kante struktur1 struktur2)))
  (if (aktive-kante-p kante)
      struktur
      (if (atom (first struktur))
	  (list (kopf kante) struktur)
	  (cons (kopf kante) struktur)))))

(defun kombiniere-strukturen (kante struktur1 struktur2)
" Es sind drei Faelle zu unterscheiden:
  (1). STRUKTUR1 ist leer  ==> STRUKTUR2
  (2). beide Teilstrukturen sind zu einer koordinierten Struktur zu verbinden
       ==> die Top-Level-Elemente von STRUKTUR1 & STRUKTUR2 sind auch die
           die Top-Level-Elemente der neuen Struktur
  (3). STRUKTUR1 ist eine 'gepackte' Struktur oder besteht aus einer Liste,
       die ein Symbol enthaelt (basierend auf einer Tilgungsregel) oder beide
       Teilstrukturen sind zu einer subordinierten Struktur zu verbinden
      ==> die neue Struktur enthaelt STRUKTUR1 & STRUKTUR2 als Top-Level-
           Elemente"
  (cond ((endp struktur1) struktur2)
        ((and (= (length (geschlossen kante)) (1+ (length struktur1)))
	      (listp (first struktur1)))
	 (append struktur1 (list struktur2)))
	(t (list struktur1 struktur2))))

;-------------------------------------------------------------------------


;;; Alternative Funktion zum Auswerten der Chart:
;;; Sie listet alle generierten Kanten auf.
; (defun auswerten (chart symbol satz)
;   (let ((i 0))
;   (dolist (k chart)
;     (format t "~a : ~a~%" i k)
;     (setf i (1+ i)))))


(defun add (pointer pointer-set)
" Diese Funktion ergaenzt die Liste der Pointer-Paare einer Kante mit einem
  weiteren Pointer-Paar."
  (let ((x (assoc (first pointer) pointer-set))) 
    (if x
	(if (member (first (second pointer)) (second x))
	    pointer-set
	    (cons (list (first pointer) (append (second pointer) (second x)))
		   (remove x pointer-set :test #'equal)))
	(cons pointer pointer-set))))


