;------------------------------------------------------------------------------
;
;                      Ein einfacher top-down Parser
;                            mit Backtracking
; 
;------------------------------------------------------------------------------


;;;;                   INFOS & INITIALISIERUNG
;;;;                   -----------------------

;;;; File : td-3.lsp

;;;; Waehle eine geeignete Syntax und ein geeignetes Lexikon:
(use :syntax syntax11 :lexikon lexikon1)

;;;; Lade (wenn noetig) die zur Strukturgenerierung notwendigen 
;;;; Funktionen:
(unless (fboundp 'transform)
   (lade-datei "td-tools.lsp"))





;;;;                        PROGRAMM
;;;;                        --------



; A. Iterative Version

(defun parse_tdi (satz &optional (symbol `(,(startsymbol))))
"Iterative Version des top-down Parsingalgorithmus mit Backtracking.
 Besonderheit: Findet nur die erste Analyse des Satzes."
  (let ((wort 0) (ableitung symbol) (stack ()) (struktur ()))
    (loop
     (show-information :wort wort :ableitung ableitung :struktur struktur
		       :stack stack)
     (if (and (eql wort (length satz)) (endp ableitung))
	 (return (transform struktur)))
     (if (or (endp ableitung) (lexkat-p (first ableitung)))
	 (if (endp stack)
	     (return nil)
	     (let ((backtrack-values (rest (backtrack satz stack))))
	       (setf ableitung (first backtrack-values)
		     wort      (second  backtrack-values)
		     struktur  (third  backtrack-values)
 		     stack     (fourth backtrack-values))))
	 (let ((continue-values
		(rest (continue satz wort ableitung struktur stack))))
	   (setf ableitung (first continue-values)
		 wort      (second  continue-values)
		 struktur  (third  continue-values)
		 stack     (fourth continue-values)))))))




; B. Rekursive Version

(defun parse_tdr (satz &optional (ableitung `(,(startsymbol)))
		       (wort 0) struktur stack)
"Rekursive Version des top-down Parsingalgorithmus mit Backtracking.
 Besonderheit: Findet alle Analysen des Satzes."
  (show-information :wort wort :ableitung ableitung :struktur struktur
		    :stack stack)
  (if (and (eql wort (length satz)) (endp ableitung))
      (progn (print (transform struktur))
	     (when (not (endp stack))
                   (apply #'parse_tdr (backtrack satz stack))))
      (if (or (endp ableitung) (lexkat-p (first ableitung)))
	  (if (endp stack)
	      nil
	      (apply #'parse_tdr (backtrack satz stack)))
	  (apply #'parse_tdr
		 (continue satz wort ableitung struktur stack)))))

;;; zusaetzlicher Wert: SATZ
(defun continue (satz wort ableitung struktur stack)
  (format t "Expandiere und reduziere ...~%")
  (let* ((regelseiten
          (expansionen (first ableitung)))
         (n-ableitung
	  (append (first regelseiten) (rest ableitung)))
         (n-struktur
	  (reduce (nthcdr wort satz) n-ableitung))
	 (n-stack
          (if (rest regelseiten)
              (cons `(,wort ,ableitung ,(rest regelseiten) ,struktur) stack)
	      stack)))
    `(,satz
      ,(if n-struktur
	   (nthcdr (length n-struktur) n-ableitung)
	   n-ableitung)
      ,(+ wort (length n-struktur))
      ,(append n-struktur
	       `(,(cons (first ableitung) (first regelseiten)))
	       struktur)
      ,n-stack)))



;;; zusaetzlicher Wert: SATZ
(defun backtrack (satz stack)
  (format t "Backtracking ...~%")
  (let* ((n-wort
          (first (first stack)))
         (n-ableitung
	  (append (first (third (first stack))) (rest (second (first stack)))))
         (n-struktur
	  (reduce (nthcdr n-wort satz) n-ableitung))
         (n-stack
          (if (rest (third (first stack)))
              (subst (rest (third (first stack)))(third (first stack)) stack)
	      (rest stack))))
    `(,satz
      ,(if n-struktur
	   (nthcdr (length n-struktur) n-ableitung)
      	   n-ableitung)
      ,(+ n-wort (length n-struktur))
      ,(append n-struktur
	       `(,(cons (first (second (first stack)))
			(first (third (first stack)))))
	       (fourth (first stack)))

      ,n-stack)))
    

(defun reduce (satz ableitung)
  (if (is_a-p (first satz) (first ableitung))
      (cons `(,(first ableitung) ,(first satz))
	    (reduce (rest satz) (rest ableitung)))
      nil))
      
