;-----------------------------------------------------------------------------
;
;                      Ein einfacher top-down Generator
;                               breadth-first
;
;------------------------------------------------------------------------------

;;;; File : td-gen.lsp
;;;; letzte Aenderung: 28.1.93
;;;; untersucht alle Ableitungsmoeglichkeiten fuer einen Satz
;;;; Waehle eine geeignete Syntax und ein geeignetes Lexikon:
(use :syntax syntax11 :lexikon lexikon1)


;;;; GENERATE-TD/BF
;;;; Die Funktion akzeptiert drei optionale Parameter:
;;;; TIEFE     - (Default-Wert: 10) legt die maximale Laenge (Zahl von Regelanwendungen)
;;;;             einer Ableitung fest und garantiert eine Terminierung der Funktion bei
;;;;             Syntaxen, die rekursive Regeln enthalten.
;;;; ABLEITUNG - (Default-Wert: Startsymbol der Syntax) enthaelt die aktuelle Linkssatzform.
;;;; SAETZE    - (Default-Wert: ()) die Liste der bislang abgeleiteten Satzfragmente.
(defun generate-td/bf (&optional (tiefe 10) (ableitung `(,(startsymbol))) (saetze nil))
  (cond ((endp ableitung) (drucke saetze))                       ; Ende einer Ableitung erreicht
	((<= tiefe 0) (values))                                  ; Tiefengrenze erreicht
        (t (dolist (x (expansionen (first ableitung)) (values))  ; Ableitung fortsetzen
	       (apply #'generate-td/bf
		      (cons (1- tiefe)
			    (reduziere-bf (append x (rest ableitung)) saetze)))))))

;;;; REDUZIERE-BF
;;;; Diese Funktion entfernt fuehrende lexikalische Kategorien aus der Linkssatzform ABLEITUNG
;;;; und ergaenzt jedes Satzfragment in SAETZE mit jeder Wortform, die in *LEXIKON* unter einer
;;;; dieser Kategorien verzeichnet ist.
(defun reduziere-bf (ableitung saetze)
  (if (lexkat-p (first ableitung))
      (reduziere-bf (rest ableitung)
		    (wortformen-eintragen (finde-wortformen (first ableitung)) saetze))
      (list ableitung saetze)))


;;;; FINDE-WORTFORMEN
;;;; Fuer die lexikalische Kategorie KAT wird die Liste aller Wortformen gebildet, deren Eintrag
;;;; in *LEXIKON* KAT enthaelt.
(defun finde-wortformen (kat)
  (apply #'append
	 (mapcar #'(lambda (x)
		     (when (member kat (eintrag-kategorien (first x)))
		       (list (first x))))
		 *lexikon*)))

;;;; WORTFORMEN-EINTRAGEN
;;;; Jede Wortform aus WORTFORMLISTE wird in jedes Satzfragment aus SAETZE (am Ende)
;;;; eingefuegt.
(defun wortformen-eintragen (wortformenliste saetze)
  (if (endp saetze)
      (mapcar #'list wortformenliste)
      (mapcan #'(lambda (y)
		  (mapcar #'(lambda(x) (append x (list y)))
		      saetze))
	      wortformenliste)))

;;;; DRUCKE
;;;; Zeilenweise Ausgabe einer Liste von Saetzen.
(defun drucke (liste)
  (dolist (x liste (values))
    (print x)))

