;;;;---------------------------------------------------------------------------
;;;;
;;;;                        Generalisierter LR-Parser
;;;;                                 nach
;;;;                               M.Tomita
;;;;
;;;;---------------------------------------------------------------------------


;;;; file : tomita-p.lsp


;;;; A. Daten
;;;; --------

(defvar *syntax*)
(defvar *lexikon*)
(setf *syntax*
      '((S NP VP PP) (det n v prep) S 
	((S NP VP) (S S PP) (NP n) (NP det n) (NP NP PP) 
	 (PP prep NP) (VP v NP))))
(setf *lexikon*
      '((i (n))
	(saw (v))
	(a (det)); (n))
	(man (n))
	(with (prep))
	(telescope (n))))

(defvar *action*)
(setf *action*
      '((0 (det (s 3)) (n (s 4)))
	(1 (prep (s 6)) ($ (accept)))
	(2 (v (s 7)) (prep (s 6)))
	(3 (n (s 10)))
	(4 (v (r 3)) (prep (r 3)) ($ (r 3)))
	(5 (prep (r 2)) ($ (r 2)))
	(6 (det (s 3)) (n (s 4)))
	(7 (det (s 3)) (n (s 4)))
	(8 (prep (r 1)) ($ (r 1)))
	(9 (v (r 5)) (prep (r 5)) ($ (r 5)))
	(10 (v (r 4)) (prep (r 4)) ($ (r 4)))
	(11 (v (r 6)) (prep (r 6) (s 6)) ($ (r 6)))
	(12 (prep (r 7) (s 6)) ($ (r 7)))))

(defvar *goto*)
(setf *goto*
      '((0 (np 2) (s 1))
        (1 (pp 5))
        (2 (pp 9) (vp 8))
        (6 (np 11))
        (7 (np 12))
        (11 (pp 9))
        (12 (pp 9))))


(defvar *stack*)
(defvar *stack-zaehler*)
(defvar eingabe)
(defvar resultat)
(defvar u)
(defvar a)
(defvar r)
(defvar re)
(defvar q)


(defvar *wald*)
(defvar *wald-zaehler*)


;;;; B. Selektoren & Praedikate
;;;; --------------------------


;;;; ACTION
;;;; Selektorfunktion fuer den Action-Teil der Parsingtabelle
(defmacro action (zustand symbol)
  `(rest (assoc ,symbol (rest (assoc ,zustand *action*)))))


;;;; GOTO
;;;; Selektorfunktion fuer den Goto-Teil der Parsingtabelle
(defmacro goto (zustand symbol)
  `(second (assoc ,symbol (rest (assoc ,zustand *goto*)))))


;;;; STARTZUSTAND
;;;; Liefert den Startzustand des Parsers als Wert
(defmacro startzustand ()
  `(caar *action*))
  

;;;; TOECHTER
;;;; Liefert die Liste aller Knoten, zu denen vom Knoten
;;;; KNOTEN eine Kante fuehrt
(defmacro toechter (knoten)
  `(second (aref *stack* ,knoten)))


;;;; ETIKETT
;;;; Liefert das Etikett eines Knotens
(defun etikett (knoten)
  (first (aref *stack* knoten)))


;;;; FINDE-KNOTEN
;;;; Sucht in einer Menge von Knoten K-MENGE nach einem Knoten mit dem
;;;; Etikett NAME
(defun finde-knoten (name k-menge &optional (typ 'zustand))
  (cond ((endp k-menge) nil)
	((eq typ 'zustand)
	 (if (eq name (etikett (first k-menge)))
	     (first k-menge)
	     (finde-knoten name (rest k-menge) typ)))
	(t (if (eq name (first (aref *wald* (etikett (first k-menge)))))
	       (first k-menge)
	       (finde-knoten name (rest k-menge) typ)))))



;;;; GENERIERE-STACK-KNOTEN
;;;; Erzeugt einen neuen Zustands-/Symbolknoten inkl. Verweis auf alle
;;;; Tochterknoten (optional) mit dem Etikett k
(defmacro generiere-stack-knoten (k &optional nachfolger)
  `(setf *stack-zaehler* (1+ *stack-zaehler*)
	 (aref *stack* *stack-zaehler*) 
	 (list ,k (or ,nachfolger (list (1- *stack-zaehler*))))))

(defmacro generiere-parse-knoten (k &optional nachfolger)
  `(setf *wald-zaehler* (1+ *wald-zaehler*)
	 (aref *wald* *wald-zaehler*) 
	 (if ,nachfolger (list ,k ,nachfolger) (list ,k))))

	     



;;;; E-REGEL-P
;;;; Das Praedikat testet, ob REGEL eine Tilgungsregel ist oder nicht.
(defmacro e-regel-p (regel)
  `(endp (rest ,regel)))


;;;; ITE-REGEL
;;;; Liefert die Regel mit dem Index I von *syntax*
(defmacro ite-regel (i)
  `(nth (1- ,i) (regeln)))





;;;; C. Hauptfunktionen
;;;; ------------------


(defun tomita_parse (satz)
  (progv
      '(  eingabe                  resultat u   a    r  re  q   i s)
      `(,(kategorien-zuweisen satz)  nil   (0) nil  nil nil nil 0 ,satz)
    (initialisiere-stack)
    (dotimes (i (length eingabe) (progn (pprint *wald*) resultat))
  ;    (pprint *stack*) (pprint *wald*)(format t "~%~a~%~%" *stack-zaehler*) (read-line)
      (verarbeite_wort))))

(defun initialisiere-stack ()
  (setf *stack* (make-array 100 :initial-element nil)
	(aref *stack* 0) (list (startzustand) nil)
	*stack-zaehler* 0
	*wald* (make-array 100 :initial-element nil)
	*wald-zaehler* -1))

(defun kategorien-zuweisen (satz)
  (let ((resultat nil) (kategorien nil))
    (dolist (wort satz (append resultat '(($))))
      (if (setq kategorien (eintrag-kategorien wort))
	  (setf resultat (append resultat (list kategorien)))
	  (return (format t "~%Fuer das Wort: ~a existiert kein Lexikoneintrag!~%"
			  wort))))))


(defun verarbeite_wort ()
  (setq a u  r nil  re nil  q nil)
  (loop
 ;   (FORMAT t "Funktion : VERARBEITE_WORT~%")
  ;  (FORMAT t "A :  ~a~%U :  ~a~%R :  ~a~%Re:  ~a~%Q :  ~a~%~%" a u r re q)
    (when (and (endp a) (endp r) (endp re))
      (return (shift)))
    (cond (a (tabelle_auswerten))
	  (r (rreduce))
	  (re (e-reduce)))))


(defun tabelle_auswerten ()
  (loop
    (when (endp a) (return))
    (auswerten (pop a) (first eingabe))))

(defun auswerten (knoten kategorien)
  (dolist (kategorie kategorien)
    (dolist (aktion (action (etikett knoten) kategorie))
      (cond ((eq (first aktion) 'accept) 
	     (setq resultat (mapcar #'etikett (toechter knoten))))
	    ((eq (first aktion) 's)
	     (setq q (neue-shift-zustaende (second aktion)
					   knoten
					   kategorie)))
	    ((and (eq (first aktion) 'r)
		  (not (e-regel-p (ite-regel (second aktion)))))
	     (dolist (x (toechter knoten)) 
	       (push `(,knoten ,x ,(second aktion)) r)))
	    (t (push `(,knoten ,(second aktion)) re))))))



;;;; SHIFT
;;;; Jeder Eintrag in Q hat das Format:
;;;; <neuer-Zustand (alter-Zustand-Index*) Kategorie>
(defun shift ()
  (setq u nil)
  (dolist (eintrag q)
    (generiere-parse-knoten (third eintrag) (nth i s))
    (generiere-stack-knoten *wald-zaehler* (second eintrag))
    (generiere-stack-knoten (first eintrag))
    (push *stack-zaehler* u))
    (setq q nil
	  i (1+ i)
	  eingabe (rest eingabe)))

;;;; NEUE-SHIFT-ZUSTAENDE
;;;; Erzeugt neue Eintraege in Q
(defun neue-shift-zustaende (neu alt kat)
  (let ((eintrag (assoc neu q)))
    (if (and eintrag (eq (third eintrag) kat))
	(cons (list neu (adjoin alt (second eintrag)) kat)
	      (remove eintrag q :test #'equal))
        (cons (list neu (list alt) kat) q))))



;;;; E-REDUCE
;;;; Jeder Eintrag in Re hat das Format:
;;;; <neuer-Zustand (alter-Zustand-Index*) Regelindex>
(defun e-reduce ()
  (FORMAT t "Funktion:  REDUCE~%")
  (FORMAT t "R :  ~a~%Q :  ~a~%" r q)
  (setq u nil)
  (dolist (eintrag re)
    (generiere-parse-knoten (first (ite-regel (third eintrag))) nil)
    (addsubnode *wald-zaehler* nil)
    (generiere-stack-knoten (third eintrag) (second eintrag))
    (generiere-stack-knoten (first eintrag))
    (push *stack-zaehler* u))
  (setq re nil a u))
    
;;;; NEUE-E-REDUCE-KNOTEN
;;;; Erzeugt in Re neue Eintraege
(defun neue-e-reduce-knoten (alt regelindex)
  (let* ((n (first (ite-regel regelindex)))
	 (neu (goto (etikett alt) n))
	 (eintrag (assoc neu re)))
    (if (and eintrag (eq (third eintrag) n))
	(cons (list neu (adjoin alt (second eintrag)) n)
	      (remove eintrag re :test #'equal))
	(cons (list neu (list alt) n) re))))



;;;; REDUCE
;;;; Jeder Eintrag in R hat das Format:
;;;; <alter-Zustand-Index Symbolknoten-Vorgaenger-Index Regel-Index>
(defun rreduce ()
  (let ((n nil))
    (dolist (eintrag r)
      (setq n (first (ite-regel (third eintrag))))
      (dolist (k (reduce-knoten (list (list (second eintrag)))
				(reduce-laenge (third eintrag))))
	(dolist (paar (neue-reduce-zustaende (toechter (first k)) n))
	  (let ((v (finde-knoten (first paar) u)))
	    (cond (v (reduce1 v n paar k))
		  (t (unless (vorhanden n k a)
		       (generiere-parse-knoten n nil)
		       (addsubnode *wald-zaehler* k))
		     (generiere-stack-knoten *wald-zaehler* (second paar))
		     (generiere-stack-knoten (first paar))
		     (setq a (adjoin *stack-zaehler* a)
			   u (adjoin *stack-zaehler* u)))))))))
  (setq r nil))


(defun reduce1 (v n paar k)
  (let ((w (finde-knoten n (toechter v) 'symbol)))
    (cond ((and w (equal (toechter w) (second paar)))
	   (addsubnode (etikett w) k))
	  (t (generiere-parse-knoten n nil)
	     (addsubnode *wald-zaehler* k)
	     (generiere-stack-knoten *wald-zaehler* (second paar))
	     (setf (second (aref *stack* v))
	           (adjoin *stack-zaehler* (toechter v)))
	     (unless (member v a)
	       (dolist (kategorie (first eingabe))
		 (dolist (eintrag (action (first paar) kategorie))
		   (when (and (eq (first eintrag) 'r)
			      (not (e-regel-p (ite-regel (second eintrag)))))
		     (push (list v n (second eintrag)) r)))))))))



;;;; REDUCE-KNOTEN
;;;; Liefert die Menge aller Symbolknoten, die bei einer Reduce-Operation
;;;; im letzten Schritt noch entfernt werden muessen
(defun reduce-knoten (knoten laenge)
  (if (= laenge 0)
      knoten
      (reduce-knoten (berechne-neue-knoten knoten)
		     (- laenge 2))))

;;;; BERECHNE-NEUE-KNOTEN
;;;; Berechnet alle von den Symbolknoten SKNOTEN aus erreichbaren
;;;; Symbolknoten (Nachfolger der von ihnen dominierten Zustandsknoten)
(defun berechne-neue-knoten (sknoten)
  (let ((z-toechter nil) (s-toechter nil))
    (dolist (s sknoten)
      (setq z-toechter (union z-toechter (toechter s))))
    (dolist (z z-toechter s-toechter)
      (setq s-toechter (union s-toechter (toechter z))))))


;;;; REDUCE-LAENGE
(defun reduce-laenge (i)
  (- (* (length (rest (ite-regel i))) 2) 2))


;;;; NEUE-REDUCE-ZUSTAENDE
;;;; Berechnet eine Liste von Paaren: erstes Element = Zustand Z, zweites
;;;; Element = Liste von Zustaenden, die durch Reduktion zu Z fuehren
(defun neue-reduce-zustaende (knoten symbol)
  (let ((paare nil) (zustand nil))
    (dolist (k knoten paare)
      (if (assoc (setf zustand
		       (goto (etikett k) symbol)) paare)
	  (push k (second (assoc zustand paare)))
	  (when zustand
	        (push `(,zustand (,k)) paare))))))


(defun berechne-neue-knoten (sknoten)
  (let ((z-toechter nil) (s-toechter nil))
    (dolist (s sknoten)
      (setq z-toechter
	(append z-toechter
		(mapcar #'(lambda (x) (cons x s))
			(toechter (first s))))))
    (dolist (z z-toechter s-toechter)
      (setq s-toechter
	(append s-toechter
		(mapcar #'(lambda (x) (cons x (rest z)))
			(toechter (first z))))))))


(defun addsubnode (knoten nachfolger)
  (setf (aref *wald* knoten)
    (append (aref *wald* knoten) (list (mapcar #'etikett nachfolger)))))


(defmacro vorhanden (kategorie nachfolger knoten)
  `(member (list ,kategorie (mapcar #'etikett ,nachfolger))
	  (mapcar #'(lambda (y) (aref *wald* (etikett y)))
		  (apply #'append 
			 (mapcar #'(lambda (x) (second (aref *stack* x))) 
				 ,knoten)))
	  :test #'equal))