;------------------------------------------------------------------------------
;
;               Tools fuer die Recognizer & Parser
;
;------------------------------------------------------------------------------


;;;; A. Selektoren und Praedikate
;;;; ----------------------------

(defun syntaktische-kategorien ()
  (first *syntax*))

(defun lexikalische-kategorien ()
  (second *syntax*))

(defun startsymbol ()
  (third *syntax*))

(defun regeln ()
  (fourth *syntax*))

(defun lexikon-eintrag (wort)
  (assoc wort *lexikon*))

(defun eintrag-kategorien (wort)
  (apply #'append (rest (lexikon-eintrag wort))))

(defun synkat-p (symbol)
  (member symbol (syntaktische-kategorien)))

(defun lexkat-p (symbol)
  (member symbol (lexikalische-kategorien)))


;;;; Fuer die top-down Recognizer und Parser:

(defun element_von-p (wort kategorie)
; prueft, ob WORT Element der lexikalischen Kategorie KATEGORIE ist
  (when (lexkat-p kategorie)
        (member kategorie (eintrag-kategorien wort))))

(defun is_a-p (wort kategorie)
; prueft, ob WORT Element der lexikalischen Kategorie KATEGORIE ist
  (when (lexkat-p kategorie)
        (member kategorie (eintrag-kategorien wort))))

(defun expansionen (symbol)
; SYMBOL ist eine nicht-terminale Kategorie von SYNTAX
; als Wert wird die Liste aller rechten Regelseiten geliefert, deren linke
; Seite mit SYMBOL identisch ist
  (mapcan #'(lambda (x) (and (eq (first x) symbol) (list (rest x))))
          (regeln)))



;;;; B. Informationsfunktionen
;;;; -------------------------

(defun show-syntax ()
; Anzeige von SYNTAX auf dem Bildschirm
  (format t "~%~%Die Syntax besteht aus: ~%")
  (format t "~%(1). den ~A syntaktischen Kategorien:~%~% "
          (length (syntaktische-kategorien)))
  (show-kategorien (syntaktische-kategorien))
  (format t "~%(2). den ~A lexikalischen Kategorien:~%~% "
          (length (lexikalische-kategorien)))
  (show-kategorien (lexikalische-kategorien))
  (format t "~%(3). dem Startsymbol:    ~A       und~%" (startsymbol))
  (format t "~%(4). den ~A Regeln:~%~%" (length (regeln)))
  (show-rules))

(defun show-kategorien (set)
  (dolist (x set (terpri))
    (format t "    ~a" x)))
  
(defun show-rules ()
; Anzeige von Regeln im ueblichen Format auf dem Bildschirm
  (dolist (x (regeln))
    (format t "     ~A -> " (first x))
    (if (endp (rest x))
        (format t "e~%")
        (dolist (y (rest x) (terpri))
          (format t "~A " y)))))

(defun show-lexikon ()
; Anzeige von LEXIKON auf dem Bildschirm
  (format t "~%~%~%Das Lexikon enthaelt folgende Eintraege:~%~%")
  (format t "~5TWortform:~25TKategorie(en):~%~%") 
  (dolist (x *lexikon*)
    (show-eintrag x)))

(defun show-eintrag (x)
  (format t "~5T~A~25T" (first x))
  (dolist (y (rest x) (terpri))
    (format t "  ~A" (first y))))


(defun show-information (&key satz wort ableitung struktur stack)
; kleine Trace-Funktion fuer die Recognizer & Parser
  (when satz (format t "Satz        : ~a ~%" satz))
  (when wort (format t "Wort        : ~a ~%" wort))
  (when ableitung (format t "Ableitung   : ~a ~%" ableitung))
  (when struktur (format t "Struktur    : ~a ~%" struktur))
  (when stack (format t "Stackinhalt : ~a ~%" stack))
  (princ ".....")(read-line)
  (terpri) (terpri))

















