;------------------------------------------------------------------------------
;
;               Tools fuer unifikationsbasierte Recognizer & Parser
;
;------------------------------------------------------------------------------


; letzte Aenderung: 28.5.92
; File: uni-tool.lsp


; neue Funktionen:
; PPRINT-DAG
; PPRINT-WERT

; geaenderte Funktionen:
; REGELN

(setq *syntax1*
      '((((cat s))
	 ((cat np))
	 ((cat vp)))
	(((cat n))
	 ((cat v)))
	((cat s))
	(((((cat s) (head _X1))               ; S -> NP VP
	   ((cat np) (head _X4)) 
	   ((cat vp) (head _X1)))
	  ((_X1 ((agr _X4)))))
	 ((((cat np) (head _X2))              ; NP -> n
	   ((cat n) (head _X2)))
	  ())
	 ((((cat vp) (head _X3))              ; VP -> v
	   ((cat v) (head _X3)))
	  ())
	 )))

(setq *lexikon1*
      '((er    (   (((cat n) (head ((kas nom))))                               ())   ))
	(pennt (   (((cat v) (head ((vform norm) (per 3) (agr ((kas nom))))))  ())   ))))



;;; nicht-terminierende Syntax
;;; frei nach Shieber[85]:

(setq *syntax2*
      '((((cat s))
	 ((cat u)))
	(((cat v)))
	((cat s))
	(((((cat s))                     ; S -> U
	   ((cat u) (f a)))
	  ())
	 ((((cat u) (f _X1))             ; U -> U V
	   ((cat u) (f ((f _X1))))
	   ((cat v)))
	  ())
	 ((((cat u))                     ; U -> V
	   ((cat v)))
	  ())
	 )))

(setq *lexikon2*
      '((a    (   (((cat v))  ())   )) ))



;;; Pretty-Printer fuer DAGs:

(defun pprint-dag (kategorien bindungen)
  (format t "~%+-------------------------------------------------------------------------------+")
  (format t "~%| ~80T|")
  (dolist (kategorie kategorien)
    (dolist (pfad kategorie)
      (format t "~%|  [ ~A ~A ]~80T|"
	      (first pfad)
	      (pprint-wert (second pfad) bindungen)))
    (format t "~%| ~80T|"))
  (format t "~%+-------------------------------------------------------------------------------+"))

(defun pprint-wert (wert bindungen)
  (cond ((and (atom wert) (not (variable-p wert))) wert)
	((atom wert) (pprint-wert (second (assoc wert bindungen)) bindungen))
	(t (mapcar #'(lambda (x)
		       (list (first x) 
			     (pprint-wert (second x) bindungen)))
		   wert))))




;;;; A. Selektoren und Praedikate
;;;; ----------------------------

(defun syntaktische-kategorien ()
  (first *syntax*))

(defun lexikalische-kategorien ()
  (second *syntax*))

(defun startsymbol ()
  (third *syntax*))

(defun regeln ()
  (fourth *syntax*))

(defun lexikon-eintrag (wort)
  (assoc wort *lexikon*))

(defun eintrag-kategorien (wort)
  (apply #'append (rest (lexikon-eintrag wort))))

(defun synkat-p (symbol)
  (member symbol (syntaktische-kategorien)))

(defun lexkat-p (symbol)
  (member symbol (lexikalische-kategorien)))



;;;; B. Informationsfunktionen
;;;; -------------------------

(defun show-syntax ()
; Anzeige von SYNTAX auf dem Bildschirm
  (format t "~%~%Die Syntax besteht aus: ~%")
  (format t "~%(1). den ~A syntaktischen Kategorien:~%~% "
          (length (syntaktische-kategorien)))
  (show-kategorien (syntaktische-kategorien))
  (format t "~%(2). den ~A lexikalischen Kategorien:~%~% "
          (length (lexikalische-kategorien)))
  (show-kategorien (lexikalische-kategorien))
  (format t "~%(3). dem Startsymbol:    ~A       und~%" (startsymbol))
  (format t "~%(4). den ~A Regeln:~%~%" (length (regeln)))
  (show-rules))

(defun show-kategorien (set)
  (dolist (x set (terpri))
    (format t "    ~a" x)))
  
(defun show-rules ()
; Anzeige von Regeln im ueblichen Format auf dem Bildschirm
  (dolist (x (regeln))
    (format t "     ~A -> " (first x))
    (if (endp (rest x))
        (format t "e~%")
        (dolist (y (rest x) (terpri))
          (format t "~A " y)))))

(defun show-lexikon ()
; Anzeige von LEXIKON auf dem Bildschirm
  (format t "~%~%~%Das Lexikon enthaelt folgende Eintraege:~%~%")
  (format t "~5TWortform:~25TKategorie(en):~%~%") 
  (dolist (x *lexikon*)
    (show-eintrag x)))

(defun show-eintrag (x)
  (format t "~5T~A~25T" (first x))
  (dolist (y (rest x) (terpri))
    (format t "  ~A" (first y))))


(defun show-information (&key satz wort ableitung struktur stack)
; kleine Trace-Funktion fuer die Recognizer & Parser
  (when satz (format t "Satz        : ~a ~%" satz))
  (when wort (format t "Wort        : ~a ~%" wort))
  (when ableitung (format t "Ableitung   : ~a ~%" ableitung))
  (when struktur (format t "Struktur    : ~a ~%" struktur))
  (when stack (format t "Stackinhalt : ~a ~%" stack))
  (princ ".....")(read-line)
  (terpri) (terpri))

















